@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999, 2000
+ * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,8 +32,8 @@
  *  Software Distribution Coordinator  or  sdc@inet.no
  *  Inferno Nettverk A/S
  *  Oslo Research Park
- *  Gaustadalen 21
- *  N-0349 Oslo
+ *  Gaustadallen 21
+ *  NO-0349 Oslo
  *  Norway
  *
  * any improvements or extensions that they make and grant Inferno Nettverk A/S
@@ -44,7 +44,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: authneg.c,v 1.48 2000/04/09 10:14:41 karls Exp $";
+"$Id: authneg.c,v 1.53 2003/07/01 13:21:25 michaels Exp $";
 
 int
 negotiate_method(s, packet)
@@ -71,7 +71,7 @@ negotiate_method(s, packet)
 	/* create request packet. */
 	request[AUTH_VERSION]	= packet->req.version;
 	request[AUTH_NMETHODS]	= (unsigned char)packet->gw.state.methodc;
-	for (rc = 0; rc < packet->gw.state.methodc; ++rc)
+	for (rc = 0; rc < (int)packet->gw.state.methodc; ++rc)
 		request[AUTH_METHODS + rc] = (unsigned char)packet->gw.state.methodv[rc];
 
 	/* send list over methods we support */
