@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999, 2000, 2001
+ * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,7 +32,7 @@
  *  Software Distribution Coordinator  or  sdc@inet.no
  *  Inferno Nettverk A/S
  *  Oslo Research Park
- *  Gaustadalllllllen 21
+ *  Gaustadallen 21
  *  NO-0349 Oslo
  *  Norway
  *
@@ -41,15 +41,37 @@
  *
  */
 
+#define _NO_FUNCTION_REDIFINE
+
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: io.c,v 1.51 2001/05/08 12:03:17 michaels Exp $";
+"$Id: io.c,v 1.62 2003/07/01 13:21:29 michaels Exp $";
 
-/* this file defines the functions. */
-#undef select
-#undef close
+#if SOCKS_CLIENT && SOCKSLIBRARY_DYNAMIC
 
+#if HAVE_EXTRA_OSF_SYMBOLS
+#define sendmsg(s, msg, flags)			sys_Esendmsg(s, msg, flags)
+#else
+#define sendmsg(s, msg, flags)			sys_sendmsg(s, msg, flags)
+#endif  /* HAVE_EXTRA_OSF_SYMBOLS */
+
+#if HAVE_EXTRA_OSF_SYMBOLS
+#define recvmsg(s, msg, flags)			sys_Erecvmsg(s, msg, flags)
+#else
+#define recvmsg(s, msg, flags)			sys_recvmsg(s, msg, flags)
+#endif  /* HAVE_EXTRA_OSF_SYMBOLS */
+
+/* XXX needed on AIX apparently */
+#ifdef recvmsg_system
+#define recvmsg recvmsg_system
+#endif /* recvmsg_system */
+
+#ifdef sendmsg_system
+#define sendmsg sendmsg_system
+#endif /* sendmsg_system */
+
+#endif /* SOCKS_CLIENT && SOCKSLIBRARY_DYNAMIC */
 
 ssize_t
 readn(d, buf, nbytes, auth)
@@ -58,6 +80,7 @@ readn(d, buf, nbytes, auth)
 	size_t nbytes;
 	struct authmethod_t *auth;
 {
+	const char *function = "readn()";
 	ssize_t p;
 	size_t left = nbytes;
 
@@ -68,6 +91,18 @@ readn(d, buf, nbytes, auth)
 			if (errno == EINTR)
 				continue;
 #endif
+
+			if (errno == EAGAIN) {
+				fd_set rset;
+
+				FD_ZERO(&rset);
+				FD_SET(d, &rset);
+				if (select(d + 1, &rset, NULL, NULL, NULL) == -1)
+					swarn("%s: select()", function);
+
+				continue;
+			}
+
 			break;
 		}
 		else if (p == 0)
@@ -88,6 +123,7 @@ writen(d, buf, nbytes, auth)
 	size_t nbytes;
 	struct authmethod_t *auth;
 {
+	const char *function = "writen()";
 	ssize_t p;
 	size_t left = nbytes;
 
@@ -98,6 +134,17 @@ writen(d, buf, nbytes, auth)
 			if (errno == EINTR)
 				continue;
 #endif
+			if (errno == EAGAIN) {
+				fd_set wset;
+
+				FD_ZERO(&wset);
+				FD_SET(d, &wset);
+				if (select(d + 1, NULL, &wset , NULL, NULL) == -1)
+					swarn("%s: select()", function);
+
+				continue;
+			}
+
 			break;
 		}
 		left -= p;
@@ -170,14 +217,17 @@ socks_sendto(s, msg, len, flags, to, tolen, auth)
 
 
 ssize_t
-recvmsgn(s, msg, flags, len)
+recvmsgn(s, msg, flags)
 	int s;
 	struct msghdr *msg;
 	int flags;
-	size_t len;
 {
-	size_t left = len;
+	const char *function = "recvmsgn()";
 	ssize_t p;
+	size_t len, left;
+
+	for (p = len = 0; p < (ssize_t)msg->msg_iovlen; ++p)
+		len += msg->msg_iov[p].iov_len;
 
 	while ((p = recvmsg(s, msg, flags)) == -1 && errno == EINTR)
 #if SOCKS_SERVER
@@ -194,22 +244,19 @@ recvmsgn(s, msg, flags, len)
 		 * to get rid of, so we have to check for it ourselves and close it,
 		 * else it just gets lost in the void.
 		 */
-		size_t i;
-		int leaked;
-		caddr_t mem;
+		size_t leaked;
+		int d;
 
-		mem = msg->msg_accrights;
-		for (i = 0; i * sizeof(leaked) < msg->msg_accrightslen; ++i) {
-			memcpy(&leaked, mem, sizeof(leaked));
-			mem += sizeof(leaked);
-			close(leaked);
+		for (leaked = 0; CMSG_SPACE(leaked * sizeof(d)) < CMSG_TOTLEN(*msg); ++leaked) {
+			CMSG_GETOBJECT(d, CMSG_CONTROLDATA(*msg), leaked * sizeof(d));
+			close(d);
 		}
 	}
 #endif /* HAVE_SOLARIS_BUGS */
 
 	if (p <= 0)
 		return p;
-	left -= p;
+	left = len - p;
 
 	if (left > 0) {
 		size_t i, count, done;
@@ -227,10 +274,27 @@ recvmsgn(s, msg, flags, len)
 			const struct iovec *io = &msg->msg_iov[i];
 
 			count += io->iov_len;
-			if (count > done) {
-				if ((p = readn(s, &((char *)(io->iov_base))[io->iov_len -
-				(count - done)], count - done, NULL)) != ((ssize_t)(count - done)))
+			if (count > done) { /* didn't read all of this iovec. */
+				if ((p = readn(s,
+				&((char *)(io->iov_base))[io->iov_len - (count - done)],
+				count - done, NULL)) != ((ssize_t)(count - done))) {
+					/*
+					 * Failed to read all data, close any descriptors we
+					 * may have gotten then.
+					 */
+					size_t leaked;
+					int d;
+
+					swarn("%s: %d bytes left", function, left);
+
+					for (leaked = 0; CMSG_SPACE(leaked * sizeof(d)) < CMSG_TOTLEN(*msg);
+					++leaked) {
+						CMSG_GETOBJECT(d, CMSG_CONTROLDATA(*msg), leaked * sizeof(d));
+						close(d);
+					}
+
 					break;
+				}
 
 				left -= p;
 				done += p;
@@ -245,6 +309,72 @@ recvmsgn(s, msg, flags, len)
 	return len - left;
 }
 
+ssize_t
+sendmsgn(s, msg, flags)
+	int s;
+	const struct msghdr *msg;
+	int flags;
+{
+	const char *function = "sendmsgn()";
+	ssize_t p;
+	size_t len, left;
+
+	for (p = len = 0; p < (ssize_t)msg->msg_iovlen; ++p)
+		len += msg->msg_iov[p].iov_len;
+
+	while ((p = sendmsg(s, msg, flags)) == -1 && errno == EINTR)
+#if SOCKS_SERVER
+		;
+#else
+		return -1;
+#endif
+
+	if (p <= 0)
+		return p;
+	left = len - p;
+
+	if (left > 0) {
+		size_t i, count, done;
+
+		/*
+		 * Can't call sendmsg() again since we could be sending ancillary data,
+		 * send the elements one by one.
+		 */
+
+		SASSERTX(p >= 0);
+
+		done = p;
+		i = count = p = 0;
+		while (i < msg->msg_iovlen && left > 0) {
+			const struct iovec *io = &msg->msg_iov[i];
+
+			count += io->iov_len;
+			if (count > done) { /* didn't send all of this iovec. */
+				while ((p = writen(s,
+				&((char *)(io->iov_base))[io->iov_len - (count - done)],
+				count - done, NULL)) != ((ssize_t)(count - done))) {
+					/*
+					 * yes, we only re-try once.  What errors should we
+					 * retry again on?
+					 */
+					swarn("%s: failed on re-try", function);
+					break;
+				}
+
+				left -= p;
+				done += p;
+			}
+
+			++i;
+		}
+	}
+
+	if (left == len)
+		return p; /* nothing read. */
+	return len - left;
+}
+
+
 int
 closen(d)
 	int d;
@@ -269,12 +399,21 @@ selectn(nfds, readfds, writefds, exceptfds, timeout)
 	fd_set *exceptfds;
 	struct timeval *timeout;
 {
+	const char *function = "selectn()";
+
 	/* const */ fd_set rset = readfds	== NULL ? rset : *readfds;
 	/* const */ fd_set wset = writefds	== NULL ? wset : *writefds;
 	/* const */ fd_set eset = exceptfds	== NULL ? eset : *exceptfds;
 	/* const */ struct timeval tout = timeout == NULL ? tout : *timeout;
 	int rc;
 
+	if (timeout != NULL)
+		slog(LOG_DEBUG, "%s, tv_sec = %ld, tv_usec = %ld",
+		function, timeout->tv_sec, timeout->tv_usec);
+	else
+		slog(LOG_DEBUG, "%s, timeout = NULL", function);
+
+
 	while ((rc = select(nfds, readfds, writefds, exceptfds, timeout)) == -1
 	&& errno == EINTR) {
 		if (readfds != NULL)
