@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999, 2000, 2001
+ * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,7 +45,7 @@
 #include "config_parse.h"
 
 static const char rcsid[] =
-"$Id: sockd_request.c,v 1.148 2002/04/18 13:34:43 michaels Exp $";
+"$Id: sockd_request.c,v 1.153 2003/07/01 13:21:48 michaels Exp $";
 
 /*
  * Since it only handles one client at a time there is no possibility
@@ -196,7 +196,7 @@ recv_req(s, req)
 	fdexpect = 1;
 
 #if !HAVE_DEFECT_RECVMSG
-	SASSERT(CMSG_GETLEN(msg) == sizeof(int) * fdexpect);
+	SASSERT(CMSG_TOTLEN(msg) == CMSG_SPACE(sizeof(int) * fdexpect));
 #endif
 
 	fdreceived = 0;
@@ -470,13 +470,13 @@ dorequest(mother, request)
 			swarn("%s: setsockopt(SO_REUSEADDR)", function);
 	}
 
-	/* need to bind address so rulespermit() has a address to compare against. */
+	/* need to bind address so rulespermit() has an address to compare against.*/
 	if ((p = sockd_bind(out, &bound, 1)) != 0) {
 		/* no such luck, bind any port and let client decide if ok. */
 
 		/* LINTED pointer casts may be troublesome */
 		TOIN(&bound)->sin_port = htons(0);
-		if ((p = sockd_bind(out, &bound, 0)) != 0)
+		if ((p = bind(out, &bound, sizeof(bound))) != 0)
 			swarn("%s: bind(%s)", function, sockaddr2string(&bound, a, sizeof(a)));
 	}
 
@@ -500,7 +500,7 @@ dorequest(mother, request)
 			io.src.auth = io.control.auth = io.state.auth;
 
 			iolog(&io.rule, &io.state, OPERATION_CONNECT, &io.src.host,
-			&io.src.auth, &boundhost, &io.dst.auth, msg, strlen(msg));
+			&io.src.auth, &boundhost, &io.dst.auth, msg, 0);
 			break;
 		}
 
@@ -511,7 +511,7 @@ dorequest(mother, request)
 			io.src.auth = io.control.auth = io.state.auth;
 
 			iolog(&io.rule, &io.state, OPERATION_CONNECT, &io.src.host,
-			&io.src.auth, &io.dst.host, &io.dst.auth, msg, strlen(msg));
+			&io.src.auth, &io.dst.host, &io.dst.auth, msg, 0);
 			break;
 
 		case SOCKS_UDPASSOCIATE: {
@@ -541,7 +541,7 @@ dorequest(mother, request)
 			io.src.auth = io.control.auth = io.state.auth;
 
 			iolog(&io.rule, &io.state, OPERATION_CONNECT, &io.src.host,
-			&io.src.auth, &io.dst.host, &io.dst.auth, msg, strlen(msg));
+			&io.src.auth, &io.dst.host, &io.dst.auth, msg, 0);
 			break;
 		}
 
@@ -722,7 +722,7 @@ dorequest(mother, request)
 
 							iolog(&io.rule, &io.state, OPERATION_ABORT,
 							&io.control.host, &io.control.auth,
-							&response.host, &io.dst.auth, emsg, strlen(emsg));
+							&response.host, &io.dst.auth, emsg, 0);
 							p = -1; /* session ended. */
 							break;
 						}
@@ -846,7 +846,7 @@ dorequest(mother, request)
 
 				iolog(&bindio.rule, &bindio.state, OPERATION_CONNECT,
 				&bindio.src.host, &bindio.src.auth, &bindio.dst.host,
-				&bindio.dst.auth, msg, strlen(msg));
+				&bindio.dst.auth, msg, 0);
 
 				if (!permit) {
 					close(sv[remote]);
@@ -896,7 +896,7 @@ dorequest(mother, request)
 					/* LINTED pointer casts may be troublesome */
 					TOIN(&replyaddr)->sin_port	= htons(0);
 
-					if (sockd_bind(sv[reply], &replyaddr, 0) != 0) {
+					if (bind(sv[reply], &replyaddr, sizeof(replyaddr)) != 0) {
 						swarn("%s: bind(%s)", function,
 						sockaddr2string(&replyaddr, a, sizeof(a)));
 						break;
@@ -1072,7 +1072,7 @@ dorequest(mother, request)
 			 * bind address for receiving UDP packets so we can tell client
 			 * where to send it's packets.
 			 */
-			if (sockd_bind(clientfd, &io.src.laddr, 0) != 0) {
+			if (bind(clientfd, &io.src.laddr, sizeof(io.src.laddr)) != 0) {
 				swarn("%s: bind(%s)", function,
 				sockaddr2string(&io.src.laddr, a, sizeof(a)));
 				send_failure(request->s, &response, SOCKS_FAILURE);
